#include"face3d.h"
#include"vector3d.h"

Face3D::Face3D()
{
  P0 = P1 = P2 = NULL; material = NULL;
  drawlevel = 0;
}

Face3D::Face3D(int dl)
{
  P0 = P1 = P2 = NULL; material = NULL;
  drawlevel = dl;
}

Vector3D Face3D::CalculateNormal(void) throw(int)
{
  Vector3D p01 = *P1 - *P0;
  Vector3D p02 = *P2 - *P0;

    // La normal es el producto vectorial normalizado de dos vectores en CW
  normal = p01 ^ p02;
  normal.normalize();

    // Inicialmente asignamos la misma normal a todos los vertices
  N0 = normal;
  N1 = normal;
  N2 = normal;
  
  return normal;
}

Vector3D Face3D::CalculatePlaneEq(void) throw(int)
{
  Vector3D p01 = *P1 - *P0;
  Vector3D p02 = *P2 - *P0;

  planeeq = (p01 ^ p02) * -1.0;
  planeeq.normalize();
  
  planeeq.W(-((planeeq.X() * P2->X()) + (planeeq.Y() * P2->Y()) + -(planeeq.Z() * P2->Z())));
  planeeq.refreshd();
  
  return planeeq;
}

void Face3D::print(void)
{
  cout << "compuesta por ";
  P0->print(); cout << "  ["; N0.print(); cout << "] - ";
  P1->print(); cout << "  ["; N1.print(); cout << "] - ";
  P2->print(); cout << "  ["; N2.print(); cout << "]\n";
}

void Face3D::save(FILE *f, int nvertex, Vector3D **vertexlist, int nmat, Material3D **matlist)
{
  int i;
  for(i = 0; i < nvertex; i++) if(vertexlist[i] == P0) break; fwrite(&i, sizeof(int), 1, f);
  for(i = 0; i < nvertex; i++) if(vertexlist[i] == P1) break; fwrite(&i, sizeof(int), 1, f);
  for(i = 0; i < nvertex; i++) if(vertexlist[i] == P2) break; fwrite(&i, sizeof(int), 1, f);
  N0.save(f);
  N1.save(f);
  N2.save(f);
  planeeq.save(f);
  normal.save(f);
  if(material == NULL)
    i = -1;
  else
    for(i = 0; i < nmat; i++) if(matlist[i] == material) break;
  fwrite(&i, sizeof(int), 1, f);
  fwrite(&drawlevel, sizeof(drawlevel), 1, f);
}

void Face3D::load(FILE *f, Vector3D **vertexlist, Material3D **matlist)
{
  int i;
  fread(&i, sizeof(int), 1, f); P0 = vertexlist[i];
  fread(&i, sizeof(int), 1, f); P1 = vertexlist[i];
  fread(&i, sizeof(int), 1, f); P2 = vertexlist[i];
  N0.load(f);
  N1.load(f);
  N2.load(f);
  planeeq.load(f);
  normal.load(f);
  fread(&i, sizeof(int), 1, f);
  material = (i >= 0) ? matlist[i] : NULL;
  fread(&drawlevel, sizeof(drawlevel), 1, f);
}
