#include<string>
#include<iostream>
#include<math.h>
#include"libmodel.h"

void Parallelogram(Model3D &model, float l, float h, float d, Vector3D tr, Material3D m)
{
  l = l / 2.0;
  h = h / 2.0;
  d = d / 2.0;
    
  model.AddFace(Vector3D( l, -h, -d) + tr, Vector3D(-l, -h, -d) + tr, Vector3D( l,  h, -d) + tr, Vector3D( 0,  0, -1), m, 0);
  model.AddFace(Vector3D( l,  h, -d) + tr, Vector3D(-l, -h, -d) + tr, Vector3D(-l,  h, -d) + tr, Vector3D( 0,  0, -1), m, 0);
  
  model.AddFace(Vector3D( l, -h,  d) + tr, Vector3D( l, -h, -d) + tr, Vector3D( l,  h,  d) + tr, Vector3D( 1,  0,  0), m, 0); /**/
  model.AddFace(Vector3D( l,  h,  d) + tr, Vector3D( l, -h, -d) + tr, Vector3D( l,  h, -d) + tr, Vector3D( 1,  0,  0), m, 0);

  model.AddFace(Vector3D(-l, -h,  d) + tr, Vector3D( l, -h,  d) + tr, Vector3D(-l,  h,  d) + tr, Vector3D( 0,  0,  1), m, 0);
  model.AddFace(Vector3D(-l,  h,  d) + tr, Vector3D( l, -h,  d) + tr, Vector3D( l,  h,  d) + tr, Vector3D( 0,  0,  1), m, 0);

  model.AddFace(Vector3D(-l, -h, -d) + tr, Vector3D(-l, -h,  d) + tr, Vector3D(-l,  h, -d) + tr, Vector3D(-1,  0,  0), m, 0); /**/
  model.AddFace(Vector3D(-l,  h, -d) + tr, Vector3D(-l, -h,  d) + tr, Vector3D(-l,  h,  d) + tr, Vector3D(-1,  0,  0), m, 0);

  model.AddFace(Vector3D( l,  h,  d) + tr, Vector3D( l,  h, -d) + tr, Vector3D(-l,  h,  d) + tr, Vector3D( 0,  1,  0), m, 0); /**/
  model.AddFace(Vector3D(-l,  h,  d) + tr, Vector3D( l,  h, -d) + tr, Vector3D(-l,  h, -d) + tr, Vector3D( 0,  1,  0), m, 0);

  model.AddFace(Vector3D( l, -h, -d) + tr, Vector3D( l, -h,  d) + tr, Vector3D(-l, -h,  d) + tr, Vector3D( 0, -1,  0), m, 0);
  model.AddFace(Vector3D( l, -h, -d) + tr, Vector3D(-l, -h,  d) + tr, Vector3D(-l, -h, -d) + tr, Vector3D( 0, -1,  0), m, 0);  
}

Model3D Parallelogram(string name, float l, float h, float d, Material3D m)
{
  Model3D model(name);
  Parallelogram(model, l, h, d, Vector3D(0,0,0), m);
  
  return model;
}

Model3D Cube(string name, float a, Material3D m)
{
  Model3D cube;
  a = a / 2.0;
  cube = Parallelogram(name, a, a, a, m);
  
  return cube;
}

Model3D ConeSmooth(string name, float r, float h, int faces, Material3D m)
{
  Model3D cone;
  Face3D *face;
  int i, j;
  h = h / 2.0;
  
  Vector3D p0(0, h, 0);
  Vector3D pb(0, -h, 0);
  Vector3D p1, p2;
  for(i = 0; i < faces; i++)
  {
    p1.set(r * cos((2 * M_PI * (((float) i) + 1.0)) / ((float ) faces)), -h, r * sin((2 * M_PI * (((float) i) + 1.0)) / ((float ) faces)));
    p2.set(r * cos((2 * M_PI * (((float) i)      )) / ((float ) faces)), -h, r * sin((2 * M_PI * (((float) i)      )) / ((float ) faces)));
    face = cone.AddFace(p0, p1, p2, m, 0);
    face->CalculateNormal();
  }
  
  for(i = 0; i < cone.nvertex; i++)
  {
    Vector3D acum;
    
    acum.set(0,0,0);
    // vertexlist[i]
    for(j = 0; j < cone.nfaces; j++)
    {
      if(cone.vertexlist[i] == cone.facelist[j]->P0) acum = acum + cone.facelist[j]->N0;
      if(cone.vertexlist[i] == cone.facelist[j]->P1) acum = acum + cone.facelist[j]->N1;
      if(cone.vertexlist[i] == cone.facelist[j]->P2) acum = acum + cone.facelist[j]->N2;
    }
    acum.normalize();
    for(j = 0; j < cone.nfaces; j++)
    {
      if(cone.vertexlist[i] == cone.facelist[j]->P0) cone.facelist[j]->N0 = acum;
      if(cone.vertexlist[i] == cone.facelist[j]->P1) cone.facelist[j]->N1 = acum;
      if(cone.vertexlist[i] == cone.facelist[j]->P2) cone.facelist[j]->N2 = acum;
    }
  }

  for(i = 0; i < faces; i++)
  {
    p1.set(r * cos((2 * M_PI * (((float) i) + 1.0)) / ((float ) faces)), -h, r * sin((2 * M_PI * (((float) i) + 1.0)) / ((float ) faces)));
    p2.set(r * cos((2 * M_PI * (((float) i)      )) / ((float ) faces)), -h, r * sin((2 * M_PI * (((float) i)      )) / ((float ) faces)));
    face = cone.AddFace(pb, p2, p1, m, 0);
    face->CalculateNormal();
  }
  
  return cone;
}


Model3D ConeFlat(string name, float r, float h, int faces, Material3D m)
{
  Model3D cone;
  Face3D *face;
  int i;
  h = h / 2.0;
  
  Vector3D p0(0, h, 0);
  Vector3D pb(0, -h, 0);
  Vector3D p1, p2;
  for(i = 0; i < faces; i++)
  {
    p1.set(r * cos((2 * M_PI * (((float) i) + 1.0)) / ((float ) faces)), -h, r * sin((2 * M_PI * (((float) i) + 1.0)) / ((float ) faces)));
    p2.set(r * cos((2 * M_PI * (((float) i)      )) / ((float ) faces)), -h, r * sin((2 * M_PI * (((float) i)      )) / ((float ) faces)));
    face = cone.AddFace(p0, p1, p2, m, 0);
    face->CalculateNormal();
  }
  
  for(i = 0; i < faces; i++)
  {
    p1.set(r * cos((2 * M_PI * (((float) i) + 1.0)) / ((float ) faces)), -h, r * sin((2 * M_PI * (((float) i) + 1.0)) / ((float ) faces)));
    p2.set(r * cos((2 * M_PI * (((float) i)      )) / ((float ) faces)), -h, r * sin((2 * M_PI * (((float) i)      )) / ((float ) faces)));
    face = cone.AddFace(pb, p2, p1, m, 0);
    face->CalculateNormal();
  }
  
  return cone;
}

static void AddQuad(Model3D &md, int i, int j, Material3D m, float CuadSep)
{
  Face3D *face;
  face = md.AddFace(Vector3D(CuadSep * ((float) i - 1), 0, CuadSep * ((float) j - 1)),
                    Vector3D(CuadSep * ((float) i - 1), 0, CuadSep * ((float) j    )),
                    Vector3D(CuadSep * ((float) i    ), 0, CuadSep * ((float) j    )),
                    Vector3D(0, 1, 0), m, 0);
  face->CalculatePlaneEq();
  face = md.AddFace(Vector3D(CuadSep * ((float) i    ), 0, CuadSep * ((float) j - 1)),
                    Vector3D(CuadSep * ((float) i - 1), 0, CuadSep * ((float) j - 1)),
                    Vector3D(CuadSep * ((float) i    ), 0, CuadSep * ((float) j    )),
                    Vector3D(0, 1, 0), m, 0);
  face->CalculatePlaneEq();
}

Model3D ModelaTableroAjedrez(char *name, float AnchuraBorde, float AlturaBorde, float CuadSep, float FactorHundimiento, Material3D cblanco, Material3D cnegro, Material3D madera)
{
  Model3D md(name);
  Face3D *face;
  int i, j;

    // Dibujamos la parte con transparencias del modelo
  for(i = -3; i < 5; i++)
    for(j = -3; j < 5; j++)
      AddQuad(md, i, j, ((i & 0x01) ^ (j & 0x01)) ? cnegro : cblanco, CuadSep);
  
    // Dibujamos los bordes del tablero
  Parallelogram(md, AnchuraBorde, AlturaBorde, CuadSep*8 + AnchuraBorde*2,
                    Vector3D((CuadSep * 4) + AnchuraBorde/2, AlturaBorde * FactorHundimiento, 0), madera);
  Parallelogram(md, CuadSep*8, AlturaBorde, AnchuraBorde,
                    Vector3D(0, AlturaBorde * FactorHundimiento, (CuadSep * 4) + AnchuraBorde/2), madera);
  Parallelogram(md, AnchuraBorde, AlturaBorde, CuadSep*8 + AnchuraBorde*2,
                    Vector3D(-((CuadSep * 4) + AnchuraBorde/2), AlturaBorde * FactorHundimiento, 0), madera);
  Parallelogram(md, CuadSep*8, AlturaBorde, AnchuraBorde,
                    Vector3D(0, AlturaBorde * FactorHundimiento, -((CuadSep * 4) + AnchuraBorde/2)), madera);

    // Dibujamos el bajo del tablero
  face = md.AddFace(Vector3D(CuadSep * ((float) -4), AlturaBorde * (FactorHundimiento - 0.5), CuadSep * ((float) -4)),
                    Vector3D(CuadSep * ((float)  4), AlturaBorde * (FactorHundimiento - 0.5), CuadSep * ((float)  4)),
                    Vector3D(CuadSep * ((float) -4), AlturaBorde * (FactorHundimiento - 0.5), CuadSep * ((float)  4)),
                    Vector3D(0, -1, 0), madera, 0);
  face->CalculateNormal();
  face = md.AddFace(Vector3D(CuadSep * ((float)  4), AlturaBorde * (FactorHundimiento - 0.5), CuadSep * ((float) -4)),
                    Vector3D(CuadSep * ((float)  4), AlturaBorde * (FactorHundimiento - 0.5), CuadSep * ((float)  4)),
                    Vector3D(CuadSep * ((float) -4), AlturaBorde * (FactorHundimiento - 0.5), CuadSep * ((float) -4)),
                    Vector3D(0, -1, 0), madera, 0);
  face->CalculateNormal();

  return md;
}
