#include<math.h>
#include"vector3d.h"

Vector3D::Vector3D(float x, float y, float z)
{
  vector[0] = (x == 0) ? 0.0 : x;
  vector[1] = (y == 0) ? 0.0 : y;
  vector[2] = (z == 0) ? 0.0 : z;
  vector[3] = 1.0;
};

Vector3D::Vector3D(float x, float y, float z, float w)
{
  vector[0] = (x == 0) ? 0.0 : x;
  vector[1] = (y == 0) ? 0.0 : y;
  vector[2] = (z == 0) ? 0.0 : z;
  vector[3] = (w == 0) ? 0.0 : w;
};

Vector3D::Vector3D(float x, float y)
{
  vector[0] = (x == 0) ? 0.0 : x;
  vector[1] = (y == 0) ? 0.0 : y;
  vector[2] = 0.0;
  vector[3] = 1.0;
};

Vector3D::Vector3D()
{
  vector[0] = vector[1] = vector[2] = 0.0;
  vector[3] = 1.0;
};
    
float Vector3D::X() { return vector[0]; }
float Vector3D::Y() { return vector[1]; }
float Vector3D::Z() { return vector[2]; }
float Vector3D::W() { return vector[3]; }
float Vector3D::X(float x) { vector[0] = (x == 0) ? 0.0 : x; return x; }
float Vector3D::Y(float y) { vector[1] = (y == 0) ? 0.0 : y; return y; }
float Vector3D::Z(float z) { vector[2] = (z == 0) ? 0.0 : z; return z; }
float Vector3D::W(float w) { vector[3] = (w == 0) ? 0.0 : w; return w; }


void Vector3D::set(float x, float y, float z) 
{
  vector[0] = (x == 0) ? 0.0 : x;
  vector[1] = (y == 0) ? 0.0 : y;
  vector[2] = (z == 0) ? 0.0 : z;
  vector[3] = 1.0; 
};

void Vector3D::set(float x, float y, float z, float w)
{
  vector[0] = (x == 0) ? 0.0 : x;
  vector[1] = (y == 0) ? 0.0 : y;
  vector[2] = (z == 0) ? 0.0 : z;
  vector[3] = (w == 0) ? 0.0 : w; 
};

void Vector3D::set(Vector3D v)
{
  vector[0] = v.vector[0];
  vector[1] = v.vector[1];
  vector[2] = v.vector[2];
  vector[3] = v.vector[3];
}

void Vector3D::print(void)    { cout << "(" << vector[0]  << ", " << vector[1]  << ", " << vector[2]  << ", " << vector[3]  << ")";   }
void Vector3D::println(void)  { cout << "(" << vector[0]  << ", " << vector[1]  << ", " << vector[2]  << ", " << vector[3]  << ")\n"; }
void Vector3D::printd(void)   { cout << "(" << dvector[0] << ", " << dvector[1] << ", " << dvector[2] << ", " << dvector[3] << ")";   }
void Vector3D::printdln(void) { cout << "(" << dvector[0] << ", " << dvector[1] << ", " << dvector[2] << ", " << dvector[3] << ")\n"; }

Vector3D Vector3D::operator + (Vector3D &a) { Vector3D b(X() + a.X(), Y() + a.Y(), Z() + a.Z()); return b; }
Vector3D Vector3D::operator - (Vector3D &a) { Vector3D b(X() - a.X(), Y() - a.Y(), Z() - a.Z()); return b; }

Vector3D Vector3D::operator += (Vector3D &a) 
{
  vector[0] += a.vector[0];
  vector[1] += a.vector[1];
  vector[2] += a.vector[2];
  vector[3] += a.vector[3];
  
  return *this;
}

Vector3D Vector3D::operator ^ (Vector3D &b)
{
  Vector3D n(
             (Y() * b.Z()) - (Z() * b.Y()),
             (Z() * b.X()) - (X() * b.Z()),
             (X() * b.Y()) - (Y() * b.X())
            );
  
  return n;
}

float Vector3D::operator * (Vector3D &b)
{
  return ((X() * b.X()) + (Y() * b.Y()) + (Z() * b.Z()));
}

Vector3D Vector3D::operator * (float a) { Vector3D n(X() * a, Y() * a, Z() * a); return n; }
Vector3D Vector3D::operator / (float a) { Vector3D n(X() / a, Y() / a, Z() / a); return n; }

bool Vector3D::operator == (Vector3D &a)
{
  return ((X() == a.X()) && (Y() == a.Y()) && (Z() == a.Z()));
}

float Vector3D::modulus(void)
{
  return sqrt(X()*X() + Y()*Y() + Z()*Z());
}

void Vector3D::normalize(void) throw(int)
{
  float m = modulus();
  
  if(m == 0)
  {
    cout << "ERROR: Vector3D::normalize(): No se puede normalizar un vector de modulo 0.0!!!\n";
    throw -1;
  }
  
  set(X()/m, Y()/m, Z()/m);
}

void Vector3D::refreshd(void)
{
  int i;
  for(i = 0; i < 4; i++) dvector[i] = vector[i];
}

void Vector3D::BoundingMin(Vector3D a)
{
  register int i;
  for(i = 0; i < 3; i++)
    if(a.vector[i] < vector[i]) vector[i] = a.vector[i];
}

void Vector3D::BoundingMax(Vector3D a)
{
  register int i;
  for(i = 0; i < 3; i++)
    if(a.vector[i] > vector[i]) vector[i] = a.vector[i];
}

void Vector3D::save(FILE *f)
{
  fwrite(vector,  sizeof(vector[0]),  4, f);
  fwrite(dvector, sizeof(dvector[0]), 4, f);
}

void Vector3D::load(FILE *f)
{
  if(fread(vector,  sizeof(vector[0]),  4, f) < 4) { fprintf(stderr, "error:Vector3D::load():Fin prematuro!\n"); }
  if(fread(dvector, sizeof(dvector[0]), 4, f) < 4) { fprintf(stderr, "error:Vector3D::load():Fin prematuro!\n"); }
}
